@@ -3,7 +3,7 @@ import datetime
 import enum
 import typing
 
-from transiter import models
+from transiter.db import models
 
 
 class NULL:
@@ -40,6 +40,10 @@ class _EntitiesInSystem(View):
         return cls(count=count, _system_id=system.id)
 
 
+class AgenciesInSystem(_EntitiesInSystem):
+    pass
+
+
 class RoutesInSystem(_EntitiesInSystem):
     pass
 
@@ -52,19 +56,27 @@ class FeedsInSystem(_EntitiesInSystem):
     pass
 
 
+class TransfersInSystem(_EntitiesInSystem):
+    pass
+
+
 @dataclasses.dataclass
 class SystemLarge(System):
-    stops: StopsInSystem = NULL
-    routes: RoutesInSystem = NULL
+    agencies: AgenciesInSystem = NULL
     feeds: FeedsInSystem = NULL
-    agency_alerts: list = dataclasses.field(default_factory=list)
+    routes: RoutesInSystem = NULL
+    stops: StopsInSystem = NULL
+    transfers: TransfersInSystem = NULL
+
+
+SystemUpdateStatus = models.SystemUpdate.Status
 
 
 @dataclasses.dataclass
 class SystemUpdate(View):
     id: str
     status: models.SystemUpdate.Status
-    status_message: str
+    stack_trace: typing.Optional[typing.List[str]]
     scheduled_at: datetime.datetime
     completed_at: datetime.datetime
     system: System = NULL
@@ -74,25 +86,59 @@ class SystemUpdate(View):
         return cls(
             id=str(update.pk),
             status=update.status,
-            status_message=update.status_message,
+            stack_trace=update.status_message.splitlines()
+            if update.status_message is not None
+            else None,
             scheduled_at=update.scheduled_at,
             completed_at=update.completed_at,
         )
 
 
 @dataclasses.dataclass
-class Route(View):
-    class Status(enum.Enum):
-        NO_SERVICE = 0
-        GOOD_SERVICE = 1
-        PLANNED_SERVICE_CHANGE = 2
-        UNPLANNED_SERVICE_CHANGE = 3
-        DELAYS = 4
+class Agency(View):
+    id: str
+    name: str
+    _system_id: str
+    alerts: list = NULL
 
+    @classmethod
+    def from_model(cls, agency: models.Agency):
+        return cls(id=agency.id, name=agency.name, _system_id=agency.system.id)
+
+
+@dataclasses.dataclass
+class AgencyLarge(View):
+    id: str
+    name: str
+    url: str
+    timezone: str
+    language: str = None
+    phone: str = None
+    fare_url: str = None
+    email: str = None
+    alerts: list = NULL
+    routes: list = dataclasses.field(default_factory=list)
+
+    @classmethod
+    def from_model(cls, agency: models.Agency):
+        return cls(
+            id=agency.id,
+            name=agency.name,
+            url=agency.url,
+            timezone=agency.timezone,
+            language=agency.language,
+            phone=agency.phone,
+            fare_url=agency.fare_url,
+            email=agency.email,
+        )
+
+
+@dataclasses.dataclass
+class Route(View):
     id: str
     color: str
     _system_id: str
-    status: Status = NULL
+    alerts: typing.List["AlertSmall"] = NULL
 
     @classmethod
     def from_model(cls, route: models.Route):
@@ -110,13 +156,13 @@ class RouteLarge(View):
     url: str
     type: models.Route.Type
     _system_id: str
-    status: Route.Status
     periodicity: float
-    alerts: list
-    service_maps: list
+    agency: Agency = None
+    alerts: list = NULL
+    service_maps: list = dataclasses.field(default_factory=list)
 
     @classmethod
-    def from_model(cls, route: models.Route, status, periodicity, alerts, service_maps):
+    def from_model(cls, route: models.Route, periodicity):
         return cls(
             id=route.id,
             color=route.color,
@@ -126,10 +172,7 @@ class RouteLarge(View):
             url=route.url,
             type=route.type,
             _system_id=route.system.id,
-            status=status,
             periodicity=periodicity,
-            alerts=alerts,
-            service_maps=service_maps,
         )
 
 
@@ -138,13 +181,19 @@ class Stop(View):
     id: str
     name: str
     _system_id: str
+    distance: float = NULL
+    system: System = NULL
     service_maps: list = NULL
     parent_stop: typing.Optional["Stop"] = NULL
     child_stops: list = NULL
+    alerts: typing.List["AlertSmall"] = NULL
 
     @classmethod
-    def from_model(cls, stop: models.Stop):
-        return cls(id=stop.id, name=stop.name, _system_id=stop.system.id)
+    def from_model(cls, stop: models.Stop, show_system=False):
+        result = cls(id=stop.id, name=stop.name, _system_id=stop.system.id)
+        if show_system:
+            result.system = System.from_model(stop.system)
+        return result
 
 
 @dataclasses.dataclass
@@ -159,6 +208,8 @@ class StopLarge(View):
     parent_stop: typing.Optional["Stop"] = NULL
     child_stops: list = NULL
     directions: list = NULL
+    alerts: typing.List["AlertSmall"] = NULL
+    transfers: typing.List["Transfer"] = NULL
     stop_times: list = dataclasses.field(default_factory=list)
 
     @classmethod
@@ -173,6 +224,45 @@ class StopLarge(View):
         )
 
 
+@dataclasses.dataclass
+class Transfer(View):
+    from_stop: Stop
+    to_stop: Stop
+    type: models.Transfer.Type
+    min_transfer_time: int = None
+    distance: int = None
+
+    @classmethod
+    def from_model(cls, transfer: models.Transfer, from_stop_view, to_stop_view):
+        return cls(
+            from_stop=from_stop_view,
+            to_stop=to_stop_view,
+            type=transfer.type,
+            min_transfer_time=transfer.min_transfer_time,
+            distance=transfer.distance,
+        )
+
+
+@dataclasses.dataclass
+class TransfersConfig(View):
+    id: str
+    distance: float
+    systems: typing.List[System] = NULL
+
+    @classmethod
+    def from_model(cls, transfers_config: models.TransfersConfig):
+        return cls(
+            id=transfers_config.id,
+            distance=transfers_config.distance,
+            systems=list(map(System.from_model, transfers_config.systems)),
+        )
+
+
+@dataclasses.dataclass
+class TransfersConfigBig(TransfersConfig):
+    transfers: typing.List[Transfer] = NULL
+
+
 @dataclasses.dataclass
 class ServiceMapWithRoutes(View):
     group_id: str
@@ -185,31 +275,42 @@ class ServiceMapWithStops(View):
     stops: typing.List[Stop]
 
 
+# TODO: add a vehicle endpoint and relevant href
+@dataclasses.dataclass
+class VehicleInTrip(View):
+    id: typing.Optional[str]
+
+    @classmethod
+    def from_model(cls, vehicle: typing.Optional[models.Vehicle]):
+        if vehicle is None:
+            return None
+        return cls(id=vehicle.id)
+
+
 @dataclasses.dataclass
 class Trip(View):
     id: str
     direction_id: bool
-    start_time: datetime.datetime
-    last_update_time: datetime.datetime
-    current_status: models.Trip.TripStatus
-    current_stop_sequence: int
-    vehicle_id: str
+    started_at: datetime.datetime
+    updated_at: datetime.datetime
     _system_id: str
     _route_id: str
+    delay: int = None
+    vehicle: VehicleInTrip = NULL
     route: Route = NULL
     last_stop: Stop = NULL
     stop_times: list = NULL
+    alerts: list = NULL
 
     @classmethod
     def from_model(cls, trip: models.Trip):
         return cls(
             id=trip.id,
             direction_id=trip.direction_id,
-            start_time=trip.start_time,
-            last_update_time=trip.last_update_time,
-            current_status=trip.current_status,
-            current_stop_sequence=trip.current_stop_sequence,
-            vehicle_id=trip.vehicle_id,
+            started_at=trip.started_at,
+            updated_at=trip.updated_at,
+            delay=trip.delay,
+            vehicle=VehicleInTrip.from_model(trip.vehicle),
             _system_id=trip.route.system.id,
             _route_id=trip.route.id,
         )
@@ -291,7 +392,7 @@ class FeedUpdate(View):
     type: models.FeedUpdate.Type
     status: models.FeedUpdate.Status
     result: models.FeedUpdate.Result
-    result_message: str
+    stack_trace: typing.Optional[typing.List[str]]
     content_hash: str
     content_length: int
     completed_at: datetime.datetime
@@ -303,7 +404,9 @@ class FeedUpdate(View):
             type=feed_update.update_type,
             status=feed_update.status,
             result=feed_update.result,
-            result_message=feed_update.result_message,
+            stack_trace=feed_update.result_message.splitlines()
+            if feed_update.result_message is not None
+            else None,
             content_hash=feed_update.content_hash,
             content_length=feed_update.content_length,
             completed_at=feed_update.completed_at,
@@ -311,27 +414,71 @@ class FeedUpdate(View):
 
 
 @dataclasses.dataclass
-class AlertLarge(View):
-    id: str
-    start_time: datetime.datetime
-    end_time: datetime.datetime
-    creation_time: datetime.datetime
+class AlertMessage(View):
     header: str
     description: str
-    url: str
+    url: str = None
+    language: str = None
+
+    @classmethod
+    def from_model(cls, alert_message: models.AlertMessage):
+        return cls(
+            header=alert_message.header,
+            description=alert_message.description,
+            url=alert_message.url,
+            language=alert_message.language,
+        )
+
+
+@dataclasses.dataclass
+class AlertSmall(View):
+    id: str
     cause: models.Alert.Cause
     effect: models.Alert.Effect
 
     @classmethod
-    def from_model(cls, alert: models.Alert):
+    def from_models(cls, active_period, alert: models.Alert):
+        return cls(id=alert.id, cause=alert.cause, effect=alert.effect)
+
+
+@dataclasses.dataclass
+class AlertLarge(View):
+    id: str
+    cause: models.Alert.Cause
+    effect: models.Alert.Effect
+    active_period: "AlertActivePeriod"
+    messages: typing.List[AlertMessage] = dataclasses.field(default_factory=list)
+
+    @classmethod
+    def from_models(cls, active_period: models.AlertActivePeriod, alert: models.Alert):
         return cls(
             id=alert.id,
-            start_time=alert.start_time,
-            end_time=alert.end_time,
-            creation_time=alert.creation_time,
-            header=alert.header,
-            description=alert.description,
-            url=alert.url,
             cause=alert.cause,
             effect=alert.effect,
+            active_period=AlertActivePeriod.from_model(active_period),
+            messages=list(map(AlertMessage.from_model, alert.messages)),
         )
+
+
+@dataclasses.dataclass
+class AlertActivePeriod(View):
+    starts_at: datetime.datetime
+    ends_at: datetime.datetime
+
+    @classmethod
+    def from_model(cls, active_period: models.AlertActivePeriod):
+        return cls(starts_at=active_period.starts_at, ends_at=active_period.ends_at)
+
+
+@dataclasses.dataclass
+class _AlertsDetailValue:
+    clazz: typing.Optional[typing.Type[View]]
+    need_messages: bool
+    need_all_active_periods: bool
+
+
+class AlertsDetail(enum.Enum):
+    NONE = _AlertsDetailValue(None, False, False)
+    CAUSE_AND_EFFECT = _AlertsDetailValue(AlertSmall, False, False)
+    MESSAGES = _AlertsDetailValue(AlertLarge, True, False)
+    ALL = _AlertsDetailValue(AlertLarge, True, True)
